﻿using BPA.SAAS.KitChenManage.Application.AExternalPlatform.BaseDto;
using BPA.SAAS.KitChenManage.Application.AExternalPlatform.Service.Authorization.Dtos;
using BPA.SAAS.KitChenManage.Application.Authorization.Dtos;
using BPA.SAAS.KitChenManage.Core.Base;
using BPA.SAAS.KitChenManage.Core.Model;
using Microsoft.AspNetCore.Components.Forms;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.KitChenManage.Application.AExternalPlatform.Service.Authorization.Services
{
    public class AuthService : IAuthService, ITransient
    {
        private readonly ISqlSugarClient _db;
        public AuthService(ISqlSugarClient db)
        {
            _db = db;
        }


        /// <summary>
        /// 获取店铺授权码列表
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<List<StoreAuthKeyDto>> GetListStoreAuthKey(BaseEPDto input)
        {
           
            var res = await _db.Queryable<BPA_StoreAuthorization, BPA_Store>((a, b) => new JoinQueryInfos(
                    JoinType.Left, a.StoreId == b.Id))
                //.Where((a, b) => a.GroupId == input.TenantId)
                .OrderBy((a, b) => a.CreateAt, OrderByType.Desc)
                .Select((a, b) => new StoreAuthKeyDto
                {
                    Id = a.Id,
                    Key = a.Key,
                    StoreId = a.StoreId,
                    StoreName = b.Name
                }).ToListAsync();
            return res;
        }

        /// <summary>
        /// 获取店铺授权码
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<StoreAuthKeyDto> GetStoreAuthKey(StoreAuthDto input)
        {
            var res = await _db.Queryable<BPA_StoreAuthorization, BPA_Store>((a, b) => new JoinQueryInfos(
                    JoinType.Left, a.StoreId == b.Id))
               // .Where((a, b) => a.GroupId == input.TenantId && a.StoreId == input.StoreId)
                .OrderBy((a, b) => a.CreateAt, OrderByType.Desc)
                .Select((a, b) => new StoreAuthKeyDto
                {
                    Id = a.Id,
                    Key = a.Key,
                    StoreId = a.StoreId,
                    StoreName = b.Name
                }).FirstAsync();
            return res;
        }

        /// <summary>
        /// 添加店铺授权码
        /// </summary>
        /// <returns></returns>
        public async Task<bool> AddStoreAuthKey(StoreAuthDto inputDto)
        {
            var data = _db.Queryable<BPA_StoreAuthorization>().Where(x => x.StoreId == inputDto.StoreId).ToList();
            if (data.Count > 0)
            {
                throw Oops.Oh("店铺授权码已存在");
            }

            var res = await _db.Insertable(new BPA_StoreAuthorization()
            {
                StoreId = inputDto.StoreId,
                Key = Guid.NewGuid().ToString(),

            }).CallEntityMethod(t => t.Create()).ExecuteCommandAsync();

            return res > 0;
        }

        /// <summary>
        /// 修改店铺授权码
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public async Task<bool> UpdateStoreAuthKey(StoreAuthDto inputDto)
        {
            var data = await _db.Queryable<BPA_StoreAuthorization>().FirstAsync(x => x.StoreId == inputDto.StoreId);
            if (data == null)
            {
                throw Oops.Oh("授权信息不存在");
            }
            data.Key = Guid.NewGuid().ToString();

            return await _db.Updateable(data).ExecuteCommandHasChangeAsync();
        }


    }
}
