﻿using BPA.KitChen.GroupMeal.SqlSugar;
using BPA.SAAS.KitChenManage.Application.AExternalPlatform.BaseDto;
using BPA.SAAS.KitChenManage.Application.AExternalPlatform.Enum;
using BPA.SAAS.KitChenManage.Comm.Tool;
using BPA.SAAS.KitChenManage.Core;
using BPA.SAAS.KitChenManage.Core.Model;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.KitChenManage.Application.AExternalPlatform.Service.CheckService.Services
{
    public class CheckServices : ICheckServices, ITransient
    {
        /// <summary>
        ///检查Sign
        /// </summary>
        public async Task CheckSign<T>(T dto)
        {

            var key = GetModelValue(dto, "Key");
            var sign = GetModelValue(dto, "Sign");
            var timestamp = GetModelValue(dto, "TimeStamp");


            //检查租户
            await CheckTenant(CurrentUser.GroupId);

            //检查key
            await CheckKey(key);


            var thisSign = DtoValidator.GetSign(dto);


            if (MD5Encryption.Encrypt(thisSign).ToUpper() != sign.ToUpper())
            {
                throw Oops.Oh(ErrorCodeEnum.Code1005);
            }
        }



        /// <summary>
        /// 获取模型值
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="t"></param>
        /// <returns></returns>
        public string GetModelValue<T>(T t, string name)
        {
            string retstr = "";
            //定义PropertyInfo的List
            List<PropertyInfo> proplist = new List<PropertyInfo>();
            //遍历泛型类的每个属性加入到List里面
            Array.ForEach<PropertyInfo>(typeof(T).GetProperties(),
                p => proplist.Add(p));

            foreach (PropertyInfo prop in proplist)
            {
                if (prop.Name.ToLower() == name.ToLower())
                {
                    var data = prop.GetValue(t, null);
                    if (data != null)
                    {
                        return data.ToString();
                    }
                }
            }

            return "";
        }

        /// <summary>
        /// 检查店铺key验证
        /// </summary>
        /// <param name="key"></param>
        public async Task CheckKey(string key)
        {
            var data = await SqlSugarDb.Db.Queryable<BPA_StoreAuthorization>()
                .FirstAsync(x => x.Key == key);
            if (data == null)
            {
                throw Oops.Oh(ErrorCodeEnum.Code1004);
            }

        }

        
        /// <summary>
        /// 检查租户
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public async Task CheckTenant(string tenantId)
        {
            Dictionary<string, string> dic = new Dictionary<string, string>();
            var url = App.Configuration["baseurl"] + "api/company/companypage";
            var inputData = "{ \"current\": 1, \"pageSize\": 999 }";
            var jsonData = HttpHelper.PostData(url, inputData, Encoding.UTF8, "application/json", dic);
            var data = JsonConvert.DeserializeObject<ResultSAASManageDto<ResultTenantDataItemDto>>(jsonData);
            if (data.statusCode != 200) throw Oops.Oh(ErrorCodeEnum.Code1000);
            if (data.data.data.Count <= 0) throw Oops.Oh(ErrorCodeEnum.Code1000);
            var tenant = data.data.data.FirstOrDefault(x => x.id == tenantId);
            if (tenant == null)
            {
                throw Oops.Oh(ErrorCodeEnum.Code1000);
            }

        }
    }
}
