﻿using BPA.Models.SqlEntity.BPA_Kitchen;
using BPA.SAAS.KitChenManage.Application.AExternalPlatform.BaseDto;
using BPA.SAAS.KitChenManage.Application.AExternalPlatform1.Service.Goods.Dtos;
using BPA.SAAS.KitChenManage.Application.Device.Dtos;
using BPA.SAAS.KitChenManage.Comm.Tool;
using BPA.SAAS.KitChenManage.Core.Model;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.KitChenManage.Application.AExternalPlatform1.Service.Goods.Services
{

    [ApiDescriptionSettings("开放平台", Tag = "商品信息"), AllowAnonymous, NonUnify]
    public class Goods: IGoodsServices, ITransient,IDynamicApiController
    {

        private readonly ISqlSugarClient _db;
        public Goods(ISqlSugarClient db)
        {
            _db = db;
        }
        /// <summary>
        /// 获取设备物料信息
        /// </summary>
        /// <returns></returns>
        [HttpPost("/api/goods/Getdevicegoods")]
        public async Task<ResultGoodsAndDeviceResultDto> GetDeviceGoods(string deviceId)
        {
            var goodsId = await _db.Queryable<BPA_DeviceGoods>()
                .Where(x => x.AutoKey.ToString()==deviceId)
                .Select(x => x.GoodsId)
                .ToListAsync();
            var data = await GetDeviceGoodspost(goodsId, deviceId);
            return data;
        }


        /// <summary>
        /// 获取商品信息
        /// </summary>
        /// <param name="goodsList"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        public async Task<ResultGoodsAndDeviceResultDto> GetDeviceGoodspost(List<string> goodsList, string autoKey)
        {
            try
            {
                var result = new ResultGoodsAndDeviceResultDto { statusCode = 200, data = new GoodsAndDeviceResultDto(), succeeded = "true" };
                Dictionary<string, string> dic = new Dictionary<string, string>();
                var goodsUrl = App.Configuration["baseurl"] + "api/goods/GetGoodsListByIds";
                var goodsInputData = JsonConvert.SerializeObject(goodsList);
                var goodsJsonData = HttpHelper.PostData(goodsUrl, goodsInputData, Encoding.UTF8, "application/json", dic);
                var goodsData = JsonConvert.DeserializeObject<ResultGoodsResultDto>(goodsJsonData);
                result.data.GoodsInfoList = goodsData?.data;
                var deviceUrl = App.Configuration["baseurl"] + "api/device/getDeviceInfoByDeviceId";
                var deviceInputData = JsonConvert.SerializeObject(autoKey);
                var deviceJsonData = HttpHelper.HttpGet(deviceUrl, $"?autoKey={autoKey}", dic, "application/json");
                var deviceData = JsonConvert.DeserializeObject<ResultDeviceResultDto>(deviceJsonData);
                result.data.DeviceInfo = deviceData?.data;
                return result;
            }
            catch (Exception ex)
            {
                throw Oops.Oh("商品信息或设备信息获取失败");
            }
        }
    }
}
