﻿using BPA.Message;
using BPA.Message.IOT;
using BPA.Models.SqlEntity.BPA_Kitchen;
using BPA.SAAS.KitChenManage.Application.Device.Dtos;
using BPA.SAAS.KitChenManage.Application.Push.Comm;
using BPA.SAAS.KitChenManage.Application.Push.奶茶机_v1.Dto;
using BPA.SAAS.KitChenManage.Application.Store.Dtos;
using BPA.SAAS.KitChenManage.Comm.Const;
using BPA.SAAS.KitChenManage.Core.Base;
using BPA.SAAS.KitChenManage.Core.Model;
using Furion.LinqBuilder;
using Furion.RemoteRequest.Extensions;
using Longbow.Tasks;
using Microsoft.AspNetCore.DataProtection.KeyManagement;
using Microsoft.CodeAnalysis.Operations;
using Microsoft.Extensions.Hosting;
using MQTTnet;
using MQTTnet.Client;
using MQTTnet.Client.Subscribing;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http.Json;
using System.Reflection;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using uPLibrary.Networking.M2Mqtt;

namespace BPA.SAAS.KitChenManage.Application.Device.Services
{
    public class DevicePushRecodeService : IDevicePushRecodeService, ITransient
    {
        private readonly ISqlSugarClient _db;
        private readonly IMqttClient _mqttClient;
        CommDefaultHandle commDefaultHandle = new();
        private string BaseServerUrl = App.GetConfig<string>("baseurl");
        MqttServer mqttServer;
        public DevicePushRecodeService(ISqlSugarClient db, IMqttClient mqttClient)
        {
            _db = db;
            _mqttClient = mqttClient;
            mqttServer = new(mqttClient);
        }
        public async Task Test()
        {
            string Topic = "2222";
            var sd=await mqttServer.Publish(Topic, new PushData() { Data = "123456"}, "1");
            //return respon;
        }
        /// <summary>
        /// 分页
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<PageUtil> Page(DevicePushRecodeDtoPageInput input)
        {
            RefAsync<int> total = 0;
            var res = await _db.Queryable<BPA_DevicePushRecode>().Where(x => x.Type == input.Type)
                .WhereIF(!string.IsNullOrWhiteSpace(input.DeviceName), x => x.DeviceName.Contains(input.DeviceName))
                  .WhereIF(!string.IsNullOrWhiteSpace(input.Status), x => x.Status == Convert.ToInt32(input.Status))
                   .Select(t => new
                   {
                       CreateAt = t.CreateAt,
                       CreateBy = t.CreateBy,
                       Id = t.Id,
                       DeviceId = t.DeviceId,
                       DeviceName = t.DeviceName,
                       Type = t.Type,
                       Topic = t.Topic,
                       DataResore = t.DataResore,
                       Status = t.Status,
                       Description = t.Description,
                       MessageId = t.MessageId,
                       TopicRely = t.TopicRely,
                       DataResoreRely = t.DataResoreRely,
                       DescriptionRely = t.DescriptionRely,
                   }).OrderBy(x => x.CreateAt, OrderByType.Desc).ToPageListAsync(input.Current, input.PageSize, total);
            PageUtil util = new PageUtil()
            {
                Total = total,
                Data = res

            };
            return util;
        }
        /// <summary>
        /// 数据下发（外部调用）
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<bool> Add_alm(List<DevicePushRecodeDtoInputalm> input)
        {
            var data = input.Adapt<List<BPA_DevicePushRecode>>();
            string groupId = App.HttpContext.Request.Headers["groupId"].ToString();
            if (string.IsNullOrWhiteSpace(groupId)) throw Oops.Oh("加盟商id不能为空");
            List<string> messageids = new();
            for (int i = 0; i < input.Count; i++)
            {
                messageids.Add(input[i].MessageId);
                var DataResore = await commDefaultHandle.GetParm(input[i].Type, input[i].Data.ToString(), input[i].DeviceId, input[i].ProductName, input[i].ProductVersion, input[i].DeviceAutoKey.ToString());
                if (DataResore == null) throw Oops.Oh("解析参数失败");
                string Topic = DataResore.TopicsStr;
                if (string.IsNullOrEmpty(Topic)) throw Oops.Oh("请配置相关topic");
                //var maxmessageId=_db.Queryable<BPA_DevicePushRecode>().Max(x => x.MessageId);
                if (string.IsNullOrWhiteSpace(input[i].MessageId)) input[i].MessageId = Guid.NewGuid().ToString();
                // var messageId = Guid.NewGuid().ToString();
                data[i].Id = Guid.NewGuid().ToString();
                data[i].Topic = Topic;
                data[i].Status = 0;
                data[i].DataResore = JsonConvert.SerializeObject(DataResore.Data);
                data[i].DeviceAutoKey = input[i].DeviceAutoKey;
                data[i].Description = "下发中，等待设备响应";
                data[i].MessageId = input[i].MessageId;
                data[i].TopicRely = DataResore.TopicsStrRely;
                data[i].DescriptionRely = "等待回调";
                data[i].GroupId = groupId;
                data[i].CreateAt = DateTime.Now;
                var respush = await mqttServer.Publish(Topic, new PushData() { Data = data[i].DataResore, DeviceId = input[i].DeviceAutoKey }, input[i].MessageId);
                if (respush != "success")
                {
                    data[i].Description = respush;
                    data[i].Status = 2;
                }
                var res = await _db.Insertable(data).ExecuteCommandAsync();
               // var getGoodsAttributeurl = "http://192.168.1.17:5006/api/hirdpartypush/updatethirdpartypushrecord";//获取商品属性
                //bool status = true;
                //var jsb = "{\"messageId\":\"" + data[i].MessageId + "\",\"deviceId\":\"" + input[i].DeviceId + "\",\"isPush\":true}";
                //var resp = await getGoodsAttributeurl.SetHttpMethod(HttpMethod.Post).SetBody(jsb).PostAsStringAsync();
                //周期性任务 (10 秒后执行1次任务)
               
                //订阅回复数据下发topics
                await mqttServer.Subscribe(DataResore.TopicsStrRely);
                var callbackUrl = BaseServerUrl + "api/hirdpartypush/updatethirdpartypushrecord";//修改第三方订单的回调
                await BindMqReceived(input[i].MessageId.ToString(), groupId, callbackUrl);
            }
            var trigger = TriggerBuilder.Default.WithInterval(TimeSpan.FromSeconds(100)).WithRepeatCount(1).Build();
            var messageid = Guid.NewGuid().ToString();
            TaskServicesManager.GetOrAdd(messageid, async (provider, token) => {
                await CallBackDevicePushRecode_alm(messageids, groupId);
            }, trigger);
            return true;
        }
        /// <summary>
        /// 添加
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<bool> Add(DevicePushRecodeDtoInput input)
        {
            var data = input.Adapt<BPA_DevicePushRecode>();
            var groupId = App.User?.FindFirst(ClaimConst.GroupId)?.Value;
          
                var DataResore = await commDefaultHandle.GetParm(input.Type, input.Data.ToString(), input.DeviceId, input.ProductName, input.ProductVersion, input.DeviceAutoKey.ToString());
                if (DataResore == null) throw Oops.Oh("解析参数失败");
                string Topic = DataResore.TopicsStr;
                if (string.IsNullOrEmpty(Topic)) throw Oops.Oh("请配置相关topic");
                //var maxmessageId=_db.Queryable<BPA_DevicePushRecode>().Max(x => x.MessageId);
               // if (string.IsNullOrWhiteSpace(input.MessageId)) input.MessageId = Guid.NewGuid().ToString();
                var messageId = Guid.NewGuid().ToString();
                data.Topic = Topic;
                data.Status = 0;
                data.DataResore = JsonConvert.SerializeObject(DataResore.Data);
                data.DeviceAutoKey = input.DeviceAutoKey;
                data.Description = "下发中，等待设备响应";
                data.MessageId = messageId;
                data.TopicRely = DataResore.TopicsStrRely;
                data.DescriptionRely = "等待回调";
                var respush = await mqttServer.Publish(Topic, new PushData() { Data = data.DataResore, DeviceId = input.DeviceAutoKey }, messageId);
                if (respush != "success")
                {
                    data.Description = respush;
                    data.Status = 2;
                }
                var res = await _db.Insertable(data).CallEntityMethod(t => t.Create()).ExecuteCommandAsync();
              
                //周期性任务 (10 秒后执行1次任务)
                var trigger = TriggerBuilder.Default.WithInterval(TimeSpan.FromSeconds(10)).WithRepeatCount(1).Build();
                TaskServicesManager.GetOrAdd(messageId, async (provider, token) => {
                    await CallBackDevicePushRecode(messageId, groupId);
                }, trigger);
                //订阅回复数据下发topics
                await mqttServer.Subscribe(DataResore.TopicsStrRely);
                await BindMqReceived(messageId, groupId, "");
            
            return true;
        }
       
        public async Task<bool> Update(string id)
        {
            var data = _db.Queryable<BPA_DevicePushRecode>().Where(x => x.Id == id).First();
            var groupId = App.User?.FindFirst(ClaimConst.GroupId)?.Value;
            data.Status = 0;
            data.Description = "下发中，等待设备响应";
            data.DescriptionRely = "等待回调";
            data.DataResoreRely = "";
            // data.MessageId= messageId;
            var respush = await mqttServer.Publish(data.Topic, new PushData() { Data = data.DataResore, DeviceId = data.DeviceAutoKey }, data.MessageId);
            if (respush != "success")
            {
                data.Description = respush;
                data.Status = 2;
            }
            var res = await _db.Updateable(data).ExecuteCommandAsync();
            var trigger = TriggerBuilder.Default.WithInterval(TimeSpan.FromSeconds(10)).WithRepeatCount(1).Build();
            TaskServicesManager.GetOrAdd(data.MessageId.ToString(), async (provider, token) =>
            {
                await CallBackDevicePushRecode(data.MessageId.ToString(), groupId);
            }, trigger);
            await BindMqReceived(data.MessageId.ToString(), groupId,"");
            return res > 0;
        }
        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<bool> Delete(List<string> input)
        {
            try
            {
                // 查询数据库中是否存在未删除的活动信息
                var resEntitites = _db.Queryable<BPA_DevicePushRecode>().In(input).ToList();
                var res = await _db.Deleteable(resEntitites).ExecuteCommandAsync();
                return res > 0;

            }
            catch (Exception)
            {
                throw Oops.Oh("删除失败");
            }
        }
        /// <summary>
        /// 数据下发回调更新数据
        /// </summary>
        /// <param name="messageId"></param>
        /// <param name="groupId"></param>
        /// <returns></returns>
        private async Task CallBackDevicePushRecode(string messageId, string groupId)
        {
            var model = await _db.Queryable<BPA_DevicePushRecode>().Where(x => x.MessageId.ToString() == messageId.ToString() && x.GroupId == groupId).FirstAsync();
            if (model.Status == 0)
            {
                model.Status = 1;
                model.Description = "下发成功,设备未回调";
                model.DescriptionRely = "响应数据失败";
                _db.Updateable(model).ExecuteCommand();
            }
        }
        /// <summary>
        /// 数据下发回调更新数据(外部调用接口使用)
        /// </summary>
        /// <param name="messageId"></param>
        /// <param name="groupId"></param>
        /// <returns></returns>
        private async Task CallBackDevicePushRecode_alm(List<string> messageId, string groupId)
        {
            for (int i = 0; i < messageId.Count; i++)
            {
                var model = await _db.Queryable<BPA_DevicePushRecode>().Where(x => x.MessageId.ToString() == messageId[i].ToString() && x.GroupId == groupId).FirstAsync();
                if (model.Status == 0)
                {
                    model.Status = 1;
                    model.Description = "下发成功,设备未回调";
                    model.DescriptionRely = "响应数据失败";
                    _db.Updateable(model).ExecuteCommand();
                    var getGoodsAttributeurl = BaseServerUrl + "api/hirdpartypush/updatethirdpartypushrecord";//获取商品属性
                    var msg = "{\"messageId\":\"" + messageId[i] + "\",\"deviceId\":\"" + model.DeviceId + "\",\"isPush\":false}";
                    await getGoodsAttributeurl.SetHttpMethod(HttpMethod.Post).SetBody(msg).PostAsStringAsync();
                }
            }
        }
        /// <summary>
        /// 监听订阅的topic主题
        /// </summary>
        /// <param name="messageId"></param>
        /// <param name="groupId"></param>
        /// <param name="callbackUrl"></param>
        /// <returns></returns>
        private async Task BindMqReceived(string messageId,string groupId,string callbackUrl)
        {
            var status = true;
            _mqttClient.UseApplicationMessageReceivedHandler(async e =>
            {
                var model = await _db.Queryable<BPA_DevicePushRecode>().Where(x => x.MessageId.ToString() == messageId.ToString() && x.GroupId == groupId).FirstAsync();
                if (model != null && model.Status == 0)
                {
                    var str = Encoding.UTF8.GetString(e.ApplicationMessage.Payload ?? Array.Empty<byte>()).Replace("\r", "").Replace("\n", "").Replace("\t", "");
                    model.DataResoreRely = str;
                    model.Description = "下发成功，设备已回调";
                    try
                    {
                        var re = JsonConvert.DeserializeObject<ReplyBaseDto>(str);
                        if (re.MessageId != messageId.ToString())
                        {
                            model = await _db.Queryable<BPA_DevicePushRecode>().Where(x => x.MessageId.ToString() == re.MessageId && x.GroupId == groupId).FirstAsync();
                            model.DataResoreRely = str;
                            model.Description = "下发成功，设备已回调";
                        }
                        if (model != null)
                        {
                            if (re.Success)
                            {
                                model.Status = 1;
                                model.DescriptionRely = "回调成功";
                            }
                            else
                            {
                                model.Status = 1;
                                model.DescriptionRely = re.Errors;
                                status = false;
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        model.Status = 1;
                        model.Description = "下发成功";
                        model.DescriptionRely = "响应数据异常，异常信息：" + ex.Message;
                    }
                    if (!string.IsNullOrWhiteSpace(callbackUrl))
                    {
                        var msg = "{\"messageId\":\"" + messageId + "\",\"deviceId\":\"" + model.DeviceId + "\",\"isPush\":false}";
                        var sd=await callbackUrl.SetHttpMethod(HttpMethod.Post).SetBody(msg).PostAsStringAsync();
                    }
                    _db.Updateable(model).ExecuteCommand();
                }
            });
        }





        #region 设备商品

        /// <summary>
        /// 获取设备商品
        /// </summary>
        /// <param name="deviceId"></param>
        /// <returns></returns>
        public async Task<List<Goods>> GetDeviceGoods(string deviceId)
        {
            var result = new List<Goods>();

            result = await _db.Queryable<BPA_DeviceGoods>().Where(x=>x.DeviceId==deviceId)
               .Select(x => new Goods()
               {
                   GoodsId = x.GoodsId,
                   GoodsName = x.GoodsName
               }).ToListAsync();

            return result;
        }

        /// <summary>
        /// 编辑设备商品
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<bool> EditDeviceGoods(DeviceGoodsInputDto inputDto)
        {
            try
            {
                _db.Ado.BeginTran();

                //删除店铺商品
                await _db.Deleteable<BPA_DeviceGoods>().Where(it => it.DeviceId == inputDto.DeviceId).ExecuteCommandAsync();
                var list = new List<BPA_DeviceGoods>();
                foreach (var item in inputDto.GoodsList)
                {
                    list.Add(new BPA_DeviceGoods()
                    {
                        Id=Guid.NewGuid().ToString(),
                        DeviceId = inputDto.DeviceId,
                        GoodsId = item.GoodsId,
                        GoodsName = item.GoodsName,
                        AutoKey=inputDto.AutoKey
                    });
                }
                var res = await _db.Insertable(list).ExecuteCommandAsync();


                _db.Ado.CommitTran();

                return res > 0;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                return false;
            }
        }

        #endregion

    }
}
