﻿using BPA.SAAS.KitChenManage.Application.Food.Dtos;
using BPA.SAAS.KitChenManage.Application.Store.Dtos;
using BPA.SAAS.KitChenManage.Application.Store.Services;
using BPA.SAAS.KitChenManage.Comm.Tool;
using BPA.SAAS.KitChenManage.Core.Base;
using BPA.SAAS.KitChenManage.Core.Model;
using Furion.LinqBuilder;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.KitChenManage.Application.Food.Services
{
    public class FoodMenuService : IFoodMenuService, ITransient
    {
        private readonly ISqlSugarClient _db;
        public FoodMenuService(ISqlSugarClient db) 
        {
            _db=db;
        }
        /// <summary>
        /// 分页
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<PageUtil> Page(FoodMenuDtoPageInput input)
        {
            RefAsync<int> total = 0;
            var res = await _db.Queryable<BPA_FoodMenu>()
                   .WhereIF(!input.Code.IsNullOrEmpty(), t => t.Code.Contains(input.Code))
                   .WhereIF(!input.Name.IsNullOrEmpty(), t => t.Name.Contains(input.Name))

                   .Select(t => new 
                   {
                       CreateAt = t.CreateAt,
                       Id = t.Id,
                       Name = t.Name,
                       Code = t.Code,
                      
                   }).OrderBy(x => x.CreateAt, OrderByType.Desc).ToPageListAsync(input.Current, input.PageSize, total);
            PageUtil util = new PageUtil()
            {
                Total = total,
                Data = res

            };
            return util;
        }
        /// <summary>
        /// 添加
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<bool> Add(FoodMenuDtoInput input)
        {
            var Store = input.Adapt<BPA_FoodMenu>();
            // role.DataScopeType = DataScopeType.ALL;
            return await _db.Insertable(Store).CallEntityMethod(t => t.Create()).ExecuteCommandAsync() > 0;
        }
        /// <summary>
        /// 修改
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<bool> Update(FoodMenuDtoInput input)
        {
            var BPA_Food = await _db.Queryable<BPA_FoodMenu>().Where(a => a.Id == input.Id).FirstAsync();

            var Store = await _db.Queryable<BPA_FoodMenu>().Where(a => a.Id == input.Id).FirstAsync();
            Store.Name = input.Name;
            Store.Code = input.Code;
           
            return _db.Updateable(Store).ExecuteCommandHasChange();

        }
        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<bool> Delete(List<string> input)
        {
            try
            {
                // 查询数据库中是否存在未删除的活动信息
                var resEntitites = _db.Queryable<BPA_FoodMenu>().In(input).ToList();
                var res = await _db.Deleteable(resEntitites).ExecuteCommandAsync();
                return res > 0;

            }
            catch (Exception)
            {
                throw Oops.Oh("删除失败");
            }
        }
        /// <summary>
        /// 菜谱关联商品
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<bool> EditFoodMenuGoods(FoodMenuBaseDto inputDto)
        {
                //删除设置的菜谱
                await _db.Deleteable<BPA_FoodMenuGoods>()
                    .Where(x => inputDto.FoodMenuId.Contains(x.FoodMenuId)).ExecuteCommandAsync();
                //菜谱添加商品
                var data = inputDto.GoodsIds.Select(x => new BPA_FoodMenuGoods()
                {
                    FoodMenuId = inputDto.FoodMenuId,

                    GoodsId = x,
                }).ToList();
                if (data.Count > 0)
                {
                    var res = await _db.Insertable(data).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                }
            return true;

        }
        /// <summary>
        /// 查询菜谱绑定的商品
        /// </summary>
        /// <param name="foodMenuId"></param>
        /// <returns></returns>
        public async Task<List<BPA_FoodMenuGoods>> GetFoodMenuGoods(string foodMenuId)
        {
            return await _db.Queryable<BPA_FoodMenuGoods>().Where(x => x.FoodMenuId == foodMenuId).ToListAsync();
        }


        /// <summary>
        /// 获取设备物料信息
        /// </summary>
        /// <returns></returns>
        public async Task<ResultSAASManageDto> GetDeviceGoods(string deviceId)
        {


            var goodsId =await _db.Queryable<BPA_DeviceGoods>()
                .Where(x => x.DeviceId == deviceId)
                .Select(x => x.GoodsId)
                .ToListAsync();

            var data = await GetDeviceGoodspost(goodsId);

            return data;
        }


        /// <summary>
        /// 获取商品信息
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public async Task<ResultSAASManageDto> GetDeviceGoodspost(List<string> goodsList)
        {
            try
            {


                Dictionary<string, string> dic = new Dictionary<string, string>();
                var url = App.Configuration["baseurl"] + "api/goods/GetGoodsListByIds";
                var inputData = JsonConvert.SerializeObject(goodsList);
                var jsonData = HttpHelper.PostData(url, inputData, Encoding.UTF8, "application/json", dic);
                var data = JsonConvert.DeserializeObject<ResultSAASManageDto>(jsonData);
                return data;
            }
            catch (Exception ex)
            {

                throw Oops.Oh("获取商品失败");
            }
            

        }
    }
}
