﻿using BPA.SAAS.KitChenManage.Application.Device.Dtos;
using MQTTnet.Client.Subscribing;
using MQTTnet;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using MQTTnet.Client;
using uPLibrary.Networking.M2Mqtt;
using BPA.SAAS.KitChenManage.Application.Push.Comm.Dto;

namespace BPA.SAAS.KitChenManage.Application.Push.Comm
{
    public class MqttServer
    {
        private readonly IMqttClient _mqttClient;
        public MqttServer(IMqttClient mqttClient)
        {
            _mqttClient=mqttClient;
        }
        /// <summary>
        /// 发布mq（发送数据）
        /// </summary>
        /// <param name="Topic"></param>
        /// <param name="data"></param>
        /// <param name="MessageId"></param>
        /// <returns></returns>
        public async Task<string> Publish(string Topic, PushData data, string MessageId)
        {
            try
            {
                if (string.IsNullOrEmpty(Topic))
                {
                    throw Oops.Oh("订阅主题不能为空！");
                }

                // 判断客户端是否连接
                if (!_mqttClient.IsConnected)
                {
                    throw Oops.Oh("MQTT 客户端尚未连接!");
                }

                // Topic = TOPIC.GetInstance.GetBusinessTopic(x, storeInfo.FirstOrDefault(a => a.Id == item.OrgId).AutoKey) + "/" + item.AutoKey;
                BPAPackage bPAPackage = new BPAPackage
                {
                    MessageId = MessageId,// MessageID.TMC_PUSH_INGREDIENTS,
                    ClientId = data.DeviceId,
                    //ClientType = Procuct
                    MessageVersion = 0x30,
                    Timestamp = DateTime.Now,
                    Message = data
                };
                //每次下发暂停200毫秒 by 王刚 2022-06-08 测试提出修改
                // Thread.Sleep(200);
                //string aa = bPAPackage.Serialize(false);
                //var fd = bPAPackage.Serialize(false);
                var applictionmessage = new MqttApplicationMessageBuilder().WithTopic(Topic).WithPayload(bPAPackage.Serialize(false)).WithAtLeastOnceQoS().Build();
                await _mqttClient.PublishAsync(applictionmessage);
                return "success";
            }
            catch (Exception e)
            {
                return "下发错误，错误信息：" + e.Message;

            }
        }
        /// <summary>
        /// 订阅,mq
        /// </summary>
        /// <param name="topic"></param>
        /// <returns></returns>
        public async Task<string> Subscribe(string topic)
        {
            string str = "success";
            try
            {
                topic = topic.Trim();
                if (string.IsNullOrEmpty(topic))
                {
                    throw Oops.Oh("订阅主题不能为空！");
                }

                // 判断客户端是否连接
                if (!_mqttClient.IsConnected)
                {
                    throw Oops.Oh("MQTT 客户端尚未连接!");
                }

                // 设置订阅参数
                var subscribeOptions = new MqttClientSubscribeOptionsBuilder()
                     .WithTopicFilter(topic)
                     .Build();
                await _mqttClient.SubscribeAsync(subscribeOptions, CancellationToken.None);
                return str;
            }
            catch (Exception e)
            {
                return "下发错误，错误信息：" + e.Message;

            }
        }
    }
}
