﻿using BPA.SAAS.KitChenManage.Application.Store.Dtos;
using BPA.SAAS.KitChenManage.Core.Base;
using BPA.SAAS.KitChenManage.Core.Model;
using Furion.LinqBuilder;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.KitChenManage.Application.Store.Services
{
    public class StoreService: IStoreService, ITransient
    {
        private readonly ISqlSugarClient _db;
        public StoreService(ISqlSugarClient db)
        {
            _db=db;
        }
        /// <summary>
        /// 分页
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<PageUtil> Page(StoreDtoPageInput input)
        {
            RefAsync<int> total = 0;
            var res = await _db.Queryable<BPA_Store>()
                   .WhereIF(!input.OrgName.IsNullOrEmpty(), t => t.OrgId.Contains(input.OrgName))
                   .WhereIF(!input.Name.IsNullOrEmpty(), t => t.Name.Contains(input.Name))
                    .WhereIF(!input.Phone.IsNullOrEmpty(), t => t.Phone.Contains(input.Phone))
                   .Select(t => new StoreDtoOutput
                   {
                       CreateAt = t.CreateAt,
                       Id = t.Id,
                       Name = t.Name,
                       Phone = t.Phone,
                       Description = t.Description,
                       OrgName=t.OrgName,
                       OrgId=t.OrgId,
                       Sort = t.Sort,
                   }).OrderBy(t => t.Sort).OrderBy(x => x.CreateAt, OrderByType.Desc).ToPageListAsync(input.Current, input.PageSize, total);
            PageUtil util = new PageUtil()
            {
                Total = total,
                Data = res

            };
            return util;
        }
        public async Task<List<StoreDtoOutput>> GetStoreList()
        {
            RefAsync<int> total = 0;
            var res = await _db.Queryable<BPA_Store>()
                   .Select(t => new StoreDtoOutput
                   {
                       CreateAt = t.CreateAt,
                       Id = t.Id,
                       Name = t.Name,
                       Phone = t.Phone,
                       Description = t.Description,
                       OrgName = t.OrgName,
                       OrgId = t.OrgId,
                       Sort = t.Sort,
                   }).OrderBy(t => t.Sort).OrderBy(x => x.CreateAt, OrderByType.Desc).ToListAsync();
           
            return res;
        }
        public async Task<List<StoreDtoOutput>> GetStoreList_alm()
        {
            string groupId = App.HttpContext.Request.Headers["groupId"].ToString();
            if (string.IsNullOrWhiteSpace(groupId)) throw Oops.Oh("加盟商id不能为空");
            var res = await _db.Queryable<BPA_Store>().Where(x=>x.GroupId== groupId)
                   .Select(t => new StoreDtoOutput
                   {
                       CreateAt = t.CreateAt,
                       Id = t.Id,
                       Name = t.Name,
                       Phone = t.Phone,
                       Description = t.Description,
                       OrgName = t.OrgName,
                       OrgId = t.OrgId,
                       Sort = t.Sort,
                   }).OrderBy(t => t.Sort).OrderBy(x => x.CreateAt, OrderByType.Desc).ToListAsync();

            return res;
        }
        public async Task<StoreDtoOutput> GetByIdStoreList_alm(string id)
        {
            string groupId = App.HttpContext.Request.Headers["groupId"].ToString();
            if (string.IsNullOrWhiteSpace(groupId)) throw Oops.Oh("加盟商id不能为空");
            var res = await _db.Queryable<BPA_Store>().Where(x => x.GroupId == groupId && x.Id== id)
                   .Select(t => new StoreDtoOutput
                   {
                       CreateAt = t.CreateAt,
                       Id = t.Id,
                       Name = t.Name,
                       Phone = t.Phone,
                       Description = t.Description,
                       OrgName = t.OrgName,
                       OrgId = t.OrgId,
                       Sort = t.Sort,
                   }).OrderBy(t => t.Sort).OrderBy(x => x.CreateAt, OrderByType.Desc).FirstAsync();

            return res;
        }
        /// <summary>
        /// 添加
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<bool> Add(StoreDtoInput input)
        {
            var Store = input.Adapt<BPA_Store>();
            // role.DataScopeType = DataScopeType.ALL;
            return await _db.Insertable(Store).CallEntityMethod(t => t.Create()).ExecuteCommandAsync() > 0;
        }
        /// <summary>
        /// 修改
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<bool> Update(StoreDtoInput input)
        {
            var BPA_Store = await _db.Queryable<BPA_Store>().Where(a => a.Id == input.Id).FirstAsync();
         
            var Store = await _db.Queryable<BPA_Store>().Where(a => a.Id == input.Id).FirstAsync();
            Store.Name = input.Name;
            Store.OrgId = input.OrgId;
            Store.Phone = input.Phone;
            Store.Description = input.Description;
            Store.Phone = input.Phone;
            Store.Sort = input.Sort;
            Store.OrgName = input.OrgName;
            return _db.Updateable(Store).ExecuteCommandHasChange();

        }
        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<bool> Delete(List<string> input)
        {
            try
            {
                // 查询数据库中是否存在未删除的活动信息
                var resEntitites = _db.Queryable<BPA_Store>().In(input).ToList();
                var res = await _db.Deleteable(resEntitites).ExecuteCommandAsync();
                 return res > 0;
              
            }
            catch (Exception)
            {
                throw Oops.Oh("删除失败");
            }
        }
    }
}
