﻿using BPA.MQTTClient;
using BPA.SAAS.KitChenManage.Core;
using Furion;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using MQTTnet.Client.Connecting;
using MQTTnet.Client.Disconnecting;
using ProtoBuf.Meta;
using System;
using System.Threading.Tasks;

namespace BPA.SAAS.KitChenManage.Web.Core
{
    public class Startup : AppStartup
    {
        public void ConfigureServices(IServiceCollection services)
        {
            services.AddConsoleFormatter();
            services.AddJwt<JwtHandler>();

            services.AddMqttClientHostedService(op =>
            {
                op.Port = int.Parse(App.Configuration["dev1_brokerHostSettings:Port"]);
                op.Server = App.Configuration["dev1_brokerHostSettings:Host"];
                op.UserName = App.Configuration["MqttClientSettings:UserName"];
                op.Password = App.Configuration["MqttClientSettings:Password"];
                op.mqttClientConnectedHandlerDelegate = new MqttClientConnectedHandlerDelegate(async e =>
                {
                    Console.WriteLine("MQTT连接成功");
                });
                op.mqttClientDisconnectedHandlerDelegate = new MqttClientDisconnectedHandlerDelegate(async e =>
                {
                    Console.WriteLine("MQTT断开连接");
                    await Task.Delay(TimeSpan.FromSeconds(5));
                    try
                    {
                        //var options = op.Server.GetService<IMqttClientOptions>();
                        //await op.Server.GetService<IMqttClient>().ConnectAsync(options);
                    }
                    catch (global::System.Exception)
                    {


                    }
                });
                op.MqttApplicationMessageReceivedHandler = new MQTTnet.Client.Receiving.MqttApplicationMessageReceivedHandlerDelegate(async e =>
                {

                });
            });
            // services.GetService<IMqttClientOptions>();
            services.AddTaskServices();
            services.AddCorsAccessor();
            services.AddRemoteRequest();
            services.AddSqlsugarSetup(App.Configuration);
            services.AddControllers()
                    .AddInjectWithUnifyResult();
        }

        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            app.UseHttpsRedirection();

            app.UseRouting();

            app.UseCorsAccessor();

            app.UseAuthentication();
            app.UseAuthorization();

            app.UseInject(string.Empty);

            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllers();
            });
        }
    }
}